/******************************************************************************
*
*   This file is part of the software tool "Cervix Image Processing 2 (CIP2)".
*
*   Author: Christoph Bergmeir
*   Supervisor: José M. Benítez
*   Copyright (c) DiCITS Lab, Sci2s group, DECSAI, University of Granada.
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
*   MA  02110-1301, USA.
*
******************************************************************************/

#include <string>
#include <iostream>

#include <cv.h>
#include <highgui.h>

struct myEllipse{
  float ox;
  float oy;
  float r1;
  float r2;
  float theta;
};

static int cmp_func_ellipses( const void* _a, const void* _b, void* userdata )
{
  int p = *((int*)userdata);
  float a = *(((float*)_a)+p);
  float b = *(((float*)_b)+p);
  if(a<b) return -1;
  else if(a>b) return 1;

  return 0;
}

float analyzeAccumulator(CvSeq* all_points_seq, float max_dist_ox, float max_dist_oy, float max_dist_r1, 
                        float max_dist_r2, float max_dist_theta, myEllipse* max_ellipse, IplImage* color_image)
{
  int param, output_counter=0; 

  param = 0;
  cvSeqSort( all_points_seq, cmp_func_ellipses, &param);

  //myEllipse max_ellipse;
  float votes_max_ellipse=0;


  myEllipse* first_ellipse_ox = CV_GET_SEQ_ELEM( myEllipse, all_points_seq, 0 );
  float last_value_ox = first_ellipse_ox->ox;
  int last_pos_ox = 0;

  for(int pos_ox=1;pos_ox<all_points_seq->total;pos_ox++)
  {
    myEllipse* ellipse_ox = CV_GET_SEQ_ELEM( myEllipse, all_points_seq, pos_ox );
    float dist_ox = abs(ellipse_ox->ox - last_value_ox);

    if((dist_ox > max_dist_ox) || (pos_ox == all_points_seq->total-1))
    {
      if(pos_ox == all_points_seq->total-1) pos_ox++;
      CvSeq* seq_slice_ox = cvSeqSlice( all_points_seq, cvSlice(last_pos_ox, pos_ox), NULL, 0 );
      param = 1;
      cvSeqSort( seq_slice_ox, cmp_func_ellipses, &param);
      //--------------------------------------
      myEllipse* first_ellipse_oy = CV_GET_SEQ_ELEM( myEllipse, seq_slice_ox, 0 );
      float last_value_oy = first_ellipse_oy->oy;
      int last_pos_oy = 0;

      for(int pos_oy=1;pos_oy<(seq_slice_ox->total);pos_oy++)
      {
        myEllipse* ellipse_oy = CV_GET_SEQ_ELEM( myEllipse, seq_slice_ox, pos_oy );
        float dist_oy = abs(ellipse_oy->oy - last_value_oy);

        if(dist_oy > max_dist_oy || (pos_oy == seq_slice_ox->total-1))
        {
          if(pos_oy == seq_slice_ox->total-1) pos_oy++;

          CvSeq* seq_slice_oy = cvSeqSlice( seq_slice_ox, cvSlice(last_pos_oy, pos_oy), NULL, 0 );
          param = 2;
          cvSeqSort( seq_slice_oy, cmp_func_ellipses, &param);

          //std::cout << "lp_ox: " << last_pos_ox << " p_ox: " << pos_ox << "\n"
          //          << "lp_oy: " << last_pos_oy << " p_oy: " << pos_oy << "\n" 
          //          << "total slices x: " << seq_slice_ox->total << "\n"
          //          << "total slices y: " << seq_slice_oy->total << "\n\n";

            //for(int x=0; x<seq_slice_oy->total; x++)
            //{
            //  myEllipse* result = CV_GET_SEQ_ELEM( myEllipse, seq_slice_oy, x );
            //  std::cout << output_counter++ << " : " << result->ox << ", " << result->oy << ", " << result->r1 << ", " << result->r2 << ", " << result->theta << "\n";
            //}
            

          //--------------------------------------
          myEllipse* first_ellipse_r1 = CV_GET_SEQ_ELEM( myEllipse, seq_slice_oy, 0 );
          float last_value_r1 = first_ellipse_r1->r1;
          int last_pos_r1 = 0;

          for(int pos_r1=1;pos_r1<(seq_slice_oy->total);pos_r1++)
          {
            myEllipse* ellipse_r1 = CV_GET_SEQ_ELEM( myEllipse, seq_slice_oy, pos_r1 );
            float dist_r1 = abs(ellipse_r1->r1 - last_value_r1);

            if(dist_r1 > max_dist_r1 || (pos_r1 == seq_slice_oy->total-1))
            {
              if(pos_r1 == seq_slice_oy->total-1) pos_r1++;

              //std::cout << "\n";

              CvSeq* seq_slice_r1 = cvSeqSlice( seq_slice_oy, cvSlice(last_pos_r1, pos_r1), NULL, 0 );
              param = 3;
              cvSeqSort( seq_slice_r1, cmp_func_ellipses, &param);

              //for(int x=0; x<seq_slice_r1->total; x++)
              //{
              //  myEllipse* result = CV_GET_SEQ_ELEM( myEllipse, seq_slice_r1, x );
              //  std::cout << output_counter++ << " : " << result->ox << ", " << result->oy << ", " << result->r1 << ", " << result->r2 << ", " << result->theta << "\n";
              //}

              //--------------------------------------
          myEllipse* first_ellipse_r2 = CV_GET_SEQ_ELEM( myEllipse, seq_slice_r1, 0 );
          float last_value_r2 = first_ellipse_r2->r2;
          int last_pos_r2 = 0;

          for(int pos_r2=1;pos_r2<(seq_slice_r1->total);pos_r2++)
          {
            myEllipse* ellipse_r2 = CV_GET_SEQ_ELEM( myEllipse, seq_slice_r1, pos_r2 );
            float dist_r2 = abs(ellipse_r2->r2 - last_value_r2);

            if(dist_r2 > max_dist_r2 || (pos_r2 == seq_slice_r1->total-1))
            {
              if(pos_r2 == seq_slice_r1->total-1) pos_r2++;

              //std::cout << "\n";

              CvSeq* seq_slice_r2 = cvSeqSlice( seq_slice_r1, cvSlice(last_pos_r2, pos_r2), NULL, 0 );
              param = 4;
              cvSeqSort( seq_slice_r2, cmp_func_ellipses, &param);

              //for(int x=0; x<seq_slice_r2->total; x++)
              //{
              //  myEllipse* result = CV_GET_SEQ_ELEM( myEllipse, seq_slice_r2, x );
              //  std::cout << output_counter++ << " : " << result->ox << ", " << result->oy << ", " << result->r1 << ", " << result->r2 << ", " << result->theta << "\n";
              //}

              //--------------------------------------
          myEllipse* first_ellipse_theta = CV_GET_SEQ_ELEM( myEllipse, seq_slice_r2, 0 );
          float last_value_theta = first_ellipse_theta->theta;
          int last_pos_theta = 0;

          for(int pos_theta=1;pos_theta<(seq_slice_r2->total);pos_theta++)
          {
            myEllipse* ellipse_theta = CV_GET_SEQ_ELEM( myEllipse, seq_slice_r2, pos_theta );
            float dist_theta = abs(ellipse_theta->theta - last_value_theta);

            if(dist_theta > max_dist_theta || (pos_theta == seq_slice_r2->total-1))
            {
              if(pos_theta == seq_slice_r2->total-1) pos_theta++;

              //std::cout << "\n";

              CvSeq* seq_slice_theta = cvSeqSlice( seq_slice_r2, cvSlice(last_pos_theta, pos_theta), NULL, 0 );
              //param = 5;
              //cvSeqSort( seq_slice_theta, cmp_func_ellipses, &param);

              for(int x=0; x<seq_slice_theta->total; x++)
              {
                myEllipse* result = CV_GET_SEQ_ELEM( myEllipse, seq_slice_theta, x );
//                cvEllipse( color_image, cvPoint(cvRound(result->ox),cvRound(result->oy)), cvSize(cvRound(result->r1),cvRound(result->r2)), result->theta, 0, 360, CV_RGB(0,255,0), 1, 8, 0 );
                //std::cout << output_counter++ << " : " << result->ox << ", " << result->oy << ", " << result->r1 << ", " << result->r2 << ", " << result->theta << "\n";
              }

              //--------------------------------------

                //max_ellipse = CV_GET_SEQ_ELEM( myEllipse, seq_slice_theta, seq_slice_theta->total/2 );

                myEllipse* tmp_el;
                myEllipse cand_el;

                param = 0;
                cvSeqSort( seq_slice_theta, cmp_func_ellipses, &param);
                tmp_el = CV_GET_SEQ_ELEM( myEllipse, seq_slice_theta, seq_slice_theta->total/2 );
                cand_el.ox = tmp_el->ox;

                param = 1;
                cvSeqSort( seq_slice_theta, cmp_func_ellipses, &param);
                tmp_el = CV_GET_SEQ_ELEM( myEllipse, seq_slice_theta, seq_slice_theta->total/2 );
                cand_el.oy = tmp_el->oy;

                param = 2;
                cvSeqSort( seq_slice_theta, cmp_func_ellipses, &param);
                tmp_el = CV_GET_SEQ_ELEM( myEllipse, seq_slice_theta, seq_slice_theta->total/2 );
                cand_el.r1 = tmp_el->r1;

                param = 3;
                cvSeqSort( seq_slice_theta, cmp_func_ellipses, &param);
                tmp_el = CV_GET_SEQ_ELEM( myEllipse, seq_slice_theta, seq_slice_theta->total/2 );
                cand_el.r2 = tmp_el->r2;

                param = 4;
                cvSeqSort( seq_slice_theta, cmp_func_ellipses, &param);
                tmp_el = CV_GET_SEQ_ELEM( myEllipse, seq_slice_theta, seq_slice_theta->total/2 );
                cand_el.theta = tmp_el->theta;

                //divide by something like the area
                //  float ellipse_fitness = seq_slice_theta->total / (cand_el.r1*cand_el.r2);

                //divide by something like the perimeter
                //float ellipse_fitness = seq_slice_theta->total / sqrt(cand_el.r1*cand_el.r1+cand_el.r2*cand_el.r2);
                //float ellipse_fitness = seq_slice_theta->total / (cand_el.r1+cand_el.r2);
                //float ellipse_fitness = seq_slice_theta->total * cand_el.r1 / (cand_el.r1-cand_el.r2);
                //float ellipse_fitness = seq_slice_theta->total / ((cand_el.r1-cand_el.r2) * cand_el.r1 * cand_el.r2);
                //float ellipse_fitness = seq_slice_theta->total / (sqrt(cand_el.r1*cand_el.r1-cand_el.r2*cand_el.r2) * sqrt(cand_el.r1*cand_el.r1+cand_el.r2*cand_el.r2));
                float ellipse_fitness = seq_slice_theta->total;// / sqrt(cand_el.r1*cand_el.r1+cand_el.r2*cand_el.r2);

                if(ellipse_fitness>votes_max_ellipse)
                {
                  votes_max_ellipse = ellipse_fitness;
                  max_ellipse->ox = cand_el.ox;
                  max_ellipse->oy = cand_el.oy;
                  max_ellipse->r1 = cand_el.r1;
                  max_ellipse->r2 = cand_el.r2;
                  max_ellipse->theta = cand_el.theta;
                }
              //std::cout << "total elements in slice: " << seq_slice_theta->total << "\n";
              //myEllipse* result = CV_GET_SEQ_ELEM( myEllipse, seq_slice_theta, seq_slice_theta->total/2 );
              //std::cout << "median element: " << result->ox << ", " << result->oy << ", " << result->r1 << ", " << result->r2 << ", " << result->theta << "\n\n";

              //--------------------------------------

              last_pos_theta = pos_theta;
              last_value_theta = ellipse_theta->theta;
            }
          }


              //--------------------------------------

              last_pos_r2 = pos_r2;
              last_value_r2 = ellipse_r2->r2;
            }
          }

              //--------------------------------------

              last_pos_r1 = pos_r1;
              last_value_r1 = ellipse_r1->r1;
            }
          }
          //--------------------------------------

          last_pos_oy = pos_oy;
          last_value_oy = ellipse_oy->oy;
        }
      }
      //--------------------------------------
      last_pos_ox = pos_ox;
      last_value_ox = ellipse_ox->ox;
    }
  }

  //std::cout << "Number of ellipses in Accumulator: " << output_counter << "\n";
  return votes_max_ellipse;
}
