/******************************************************************************
*
*   This file is part of the software tool "Cervix Image Processing 2 (CIP2)".
*
*   Author: Christoph Bergmeir
*   Supervisor: Jos M. Bentez
*   Copyright (c) DiCITS Lab, Sci2s group, DECSAI, University of Granada.
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
*   MA  02110-1301, USA.
*
******************************************************************************/

#include <cv.h>
#include <highgui.h>

int main( int argc, char** argv )
{
    IplImage* src;
    if((src=cvLoadImage(argv[1], 1))!= 0)
    {
        IplImage* h_plane = cvCreateImage( cvGetSize(src), 8, 1 );
        IplImage* s_plane = cvCreateImage( cvGetSize(src), 8, 1 );
        IplImage* v_plane = cvCreateImage( cvGetSize(src), 8, 1 );
        IplImage* planes[] = { h_plane, s_plane };
        IplImage* hsv = cvCreateImage( cvGetSize(src), 8, 3 );
//        int h_bins = 30, s_bins = 32;
        int h_bins = 180, s_bins = 255;
        int hist_size[] = {h_bins, s_bins};
        float h_ranges[] = { 0, 180 }; /* hue varies from 0 (~0red) to 180 (~360red again) */
        float s_ranges[] = { 0, 255 }; /* saturation varies from 0 (black-gray-white) to 255 (pure spectrum color) */
        float* ranges[] = { h_ranges, s_ranges };
//        int scale = 10;
        int scale = 10;
        IplImage* hist_img = cvCreateImage( cvSize(h_bins*scale,s_bins*scale), 8, 3 );
        CvHistogram* hist;
        float max_value = 0;
        int h, s;

        cvCvtColor( src, hsv, CV_BGR2HSV );
        cvCvtPixToPlane( hsv, h_plane, s_plane, v_plane, 0 );
        hist = cvCreateHist( 2, hist_size, CV_HIST_ARRAY, ranges, 1 );
        cvCalcHist( planes, hist, 0, 0 );
        cvGetMinMaxHistValue( hist, 0, &max_value, 0, 0 );
        cvZero( hist_img );

        for( h = 0; h < h_bins; h++ )
        {
            for( s = 0; s < s_bins; s++ )
            {
                float bin_val = cvQueryHistValue_2D( hist, h, s );
                //int intensity = cvRound(bin_val*255/max_value);
                int intensity = cvRound(log((double)bin_val)*255/log((double)max_value));
                cvRectangle( hist_img, cvPoint( h*scale, s*scale ),
                             cvPoint( (h+1)*scale - 1, (s+1)*scale - 1),
                             CV_RGB(intensity,intensity,intensity), /* draw a grayscale histogram.
                                                                       if you have idea how to do it
                                                                       nicer let us know */
                             CV_FILLED );
            }
        }

        cvSaveImage( argv[2], hist_img );
    }
}

/*#include <string>
#include <iostream>

#include <cv.h>
#include <highgui.h>

int main( int argc, const char * argv[] )
{
  IplImage* img,*pimg;

  img=cvLoadImage( argv[1], CV_LOAD_IMAGE_UNCHANGED );

  int roi_x = atoi( argv[3] );
  int roi_y = atoi( argv[4] );
  int width = atoi( argv[5] );
  int height = atoi( argv[6] );

  pimg = cvCreateImage( cvSize(width,height), img->depth, img->nChannels );

  cvSetImageROI(img,cvRect(roi_x,roi_y,width,height)); 
  cvCopy(img,pimg); 
  cvResetImageROI(img);

  cvSaveImage( argv[2], pimg );

  if(img) cvReleaseImage(&img);
  if(pimg) cvReleaseImage(&pimg);

  return 0;
}*/
