/******************************************************************************
*
*   This file is part of the software tool "Cervix Image Processing 2 (CIP2)".
*
*   Author: Christoph Bergmeir
*   Supervisor: Jos M. Bentez
*   Copyright (c) DiCITS Lab, Sci2s group, DECSAI, University of Granada.
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
*   MA  02110-1301, USA.
*
******************************************************************************/

#include <string>
#include <iostream>

#include <cv.h>
#include <highgui.h>

int main( int argc, char** argv )
{
    IplImage* src;
    if((src=cvLoadImage(argv[1], 1))!= 0)
    {
        IplImage* r_plane = cvCreateImage( cvGetSize(src), 8, 1 );
        IplImage* g_plane = cvCreateImage( cvGetSize(src), 8, 1 );
        IplImage* b_plane = cvCreateImage( cvGetSize(src), 8, 1 );
        IplImage* nr_plane = cvCreateImage( cvGetSize(src), 8, 1 );
        IplImage* ng_plane = cvCreateImage( cvGetSize(src), 8, 1 );

        IplImage* planes[] = { nr_plane, ng_plane };
        //IplImage* hsv = cvCreateImage( cvGetSize(src), 8, 3 );
        int r_bins = 255, g_bins = 255;
        int hist_size[] = {r_bins, g_bins};
        float r_ranges[] = { 0, 255 }; /* hue varies from 0 (~0red) to 180 (~360red again) */
        float g_ranges[] = { 0, 255 }; /* saturation varies from 0 (black-gray-white) to 255 (pure spectrum color) */
        float* ranges[] = { r_ranges, g_ranges };
        int scale = 1;
        IplImage* hist_img = cvCreateImage( cvSize(r_bins*scale,g_bins*scale), 8, 3 );
        CvHistogram* hist;
        float max_value = 0;
        int r, g;

        //cvCvtColor( src, hsv, CV_BGR2HSV );
        cvCvtPixToPlane( src, r_plane, g_plane, b_plane, 0 );


    int nl= r_plane->height;
    int nc= r_plane->width;
    int step= r_plane->widthStep;

    unsigned char *r_plane_data= reinterpret_cast<unsigned char *>(r_plane->imageData);
    unsigned char *g_plane_data= reinterpret_cast<unsigned char *>(g_plane->imageData);
    unsigned char *b_plane_data= reinterpret_cast<unsigned char *>(b_plane->imageData);
    unsigned char *nr_plane_data= reinterpret_cast<unsigned char *>(nr_plane->imageData);
    unsigned char *ng_plane_data= reinterpret_cast<unsigned char *>(ng_plane->imageData);
    //std::cout << "1";
    //int num_all_points = 0;
    for (int i=0; i<nl; i++) {
      for (int j=0; j<nc; j++) {

        double sum = r_plane_data[j] + g_plane_data[j] + b_plane_data[j];
        double nr = 0;
        double ng = 0;

        if(sum != 0)
        {
          nr = (r_plane_data[j] * 255) / sum;
          ng = (g_plane_data[j] * 255) / sum;
        };

        nr_plane_data[j] = (int) nr;
        ng_plane_data[j] = (int) ng;
        //if(r_plane_data[j]>150)
        //{
        //  num_all_points++;
        //  cvSeqPush(all_points_seq, &cvPoint2D32f(j,i));
        //  //std::cout << "[j,i]: " << j << " " << i << " px+j-kernel_size/2: " << px+j-kernel_size/2 << " py+i-kernel_size/2: " << py+i-kernel_size/2 << "\n";
        //}
        //else std::cout << "missing\n";
      }
      r_plane_data+= step;
      g_plane_data+= step;
      b_plane_data+= step;
      nr_plane_data+= step;
      ng_plane_data+= step;
    }
    //std::cout << "2";

        //cvAdd( const CvArr* src1, const CvArr* src2, CvArr* dst, const CvArr* mask=NULL );
        //cvAdd( r_plane, g_plane, CvArr* dst);
        hist = cvCreateHist( 2, hist_size, CV_HIST_ARRAY, ranges, 1 );
        cvCalcHist( planes, hist, 0, 0 );
        cvGetMinMaxHistValue( hist, 0, &max_value, 0, 0 );
        cvZero( hist_img );

        for( r = 0; r < r_bins; r++ )
        {
            for( g = 0; g < g_bins; g++ )
            {
                float bin_val = cvQueryHistValue_2D( hist, r, g );
                //int intensity = cvRound(bin_val*255/max_value);
                int intensity = cvRound(log((double)bin_val)*255/log((double)max_value));
                cvRectangle( hist_img, cvPoint( r*scale, g*scale ),
                             cvPoint( (r+1)*scale - 1, (g+1)*scale - 1),
                             CV_RGB(intensity,intensity,intensity), /* draw a grayscale histogram.
                                                                       if you have idea how to do it
                                                                       nicer let us know */
                             CV_FILLED );
            }
        }

        cvSaveImage( argv[2], hist_img );
    }
}

/*#include <string>
#include <iostream>

#include <cv.h>
#include <highgui.h>

int main( int argc, const char * argv[] )
{
  IplImage* img,*pimg;

  img=cvLoadImage( argv[1], CV_LOAD_IMAGE_UNCHANGED );

  int roi_x = atoi( argv[3] );
  int roi_y = atoi( argv[4] );
  int width = atoi( argv[5] );
  int height = atoi( argv[6] );

  pimg = cvCreateImage( cvSize(width,height), img->depth, img->nChannels );

  cvSetImageROI(img,cvRect(roi_x,roi_y,width,height)); 
  cvCopy(img,pimg); 
  cvResetImageROI(img);

  cvSaveImage( argv[2], pimg );

  if(img) cvReleaseImage(&img);
  if(pimg) cvReleaseImage(&pimg);

  return 0;
}*/
