/******************************************************************************
*
*   This file is part of the software tool "Cervix Image Processing 2 (CIP2)".
*
*   Author: Christoph Bergmeir
*   Supervisor: José M. Benítez
*   Copyright (c) DiCITS Lab, Sci2s group, DECSAI, University of Granada.
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
*   MA  02110-1301, USA.
*
******************************************************************************/

#include <string>
#include <iostream>

#include <cv.h>
#include <highgui.h>

int main( int argc, const char * argv[] )
{
  IplImage* img,*pimg;

  img=cvLoadImage( argv[1], CV_LOAD_IMAGE_UNCHANGED );
  pimg = cvCreateImage( cvGetSize(img), 8, 3 );

  cvCvtColor( img, pimg, CV_GRAY2BGR );

  CvMemStorage* storage = cvCreateMemStorage(0);

  double param1 = atof( argv[3] );
  double param2 = atof( argv[4] );
  int minRadius = atoi( argv[5] );
  int maxRadius = atoi( argv[6] );

  CvSeq* circles = cvHoughCircles( img, storage, CV_HOUGH_GRADIENT, 2, img->height/4, param1, param2, minRadius, maxRadius );
  int i;
  for( i = 0; i < circles->total; i++ )
  {
       float* p = (float*)cvGetSeqElem( circles, i );
       cvCircle( pimg, cvPoint(cvRound(p[0]),cvRound(p[1])), 3, CV_RGB(0,255,0), -1, 8, 0 );
       cvCircle( pimg, cvPoint(cvRound(p[0]),cvRound(p[1])), cvRound(p[2]), CV_RGB(255,0,0), 3, 8, 0 );
  }

  cvSaveImage( argv[2], pimg );

  if(img) cvReleaseImage(&img);
  if(pimg) cvReleaseImage(&pimg);

  return 0;
}
