/******************************************************************************
*
*   This file is part of the software tool "Cervix Image Processing 2 (CIP2)".
*
*   Author: Christoph Bergmeir
*   Supervisor: José M. Benítez
*   Copyright (c) DiCITS Lab, Sci2s group, DECSAI, University of Granada.
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
*   MA  02110-1301, USA.
*
******************************************************************************/

//#include <string>
#include <iostream>
#include <sstream>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <cv.h>
#include <highgui.h>
#include <math.h>

typedef struct MyLine
{
  float vx,vy,px,py;
}
MyLine;

CV_INLINE  MyLine  myLine( float vx, float vy, float px, float py )
{
  MyLine l;

  l.vx = vx;
  l.vy = vy;
  l.px = px;
  l.py = py;

  return l;
}

int main(int argc, char** argv)
{
  IplImage* src_image;
  if((src_image=cvLoadImage(argv[1], 0))!= 0)
  {
    int roi_size = atoi( argv[3] );
    int roi_x = atoi( argv[4] );
    int roi_y = atoi( argv[5] );
    int samples_num = atoi( argv[6] );
    int kernel_size = atoi( argv[7] );
    double min_dist = atof( argv[8] );
    double max_dist = atof( argv[9] );
    double ray_vote = max_dist/2;

    IplImage* roi = cvCreateImage( cvSize(roi_size,roi_size), 8, 1 );
    cvSetImageROI(src_image,cvRect(roi_x,roi_y,roi_size,roi_size)); 
    cvCopy(src_image,roi); 
    cvResetImageROI(src_image);

    IplImage* color_roi = cvCreateImage( cvGetSize(roi), 8, 3 );
    cvCvtColor( roi, color_roi, CV_GRAY2BGR );

    CvMemStorage* all_points_storage = cvCreateMemStorage(0);

    CvSeq* all_points_seq = cvCreateSeq( CV_32FC2, sizeof(CvSeq), sizeof(CvPoint2D32f), all_points_storage );

    int nl= roi->height;
    int nc= roi->width;
    int step= roi->widthStep;

    unsigned char *roi_data= reinterpret_cast<unsigned char *>(roi->imageData);

    int num_all_points = 0;
    for (int i=0; i<nl; i++) {
      for (int j=0; j<nc; j++) {
        if(roi_data[j]>150)
        {
          num_all_points++;
          cvSeqPush(all_points_seq, &cvPoint2D32f(j,i));
          //std::cout << "[j,i]: " << j << " " << i << " px+j-kernel_size/2: " << px+j-kernel_size/2 << " py+i-kernel_size/2: " << py+i-kernel_size/2 << "\n";
        }
        //else std::cout << "missing\n";
      }
      roi_data+= step;
    }

    CvMat temp_mat;
    IplImage* accum_image = cvCreateImage( cvSize(roi_size,roi_size), 8, 1 );
    cvSet(accum_image, cvScalar(0));
    CvMat* accum = cvGetMat( accum_image, &temp_mat);
    uchar *accum_data = accum->data.ptr;

    srand ( time(NULL) );
    //    samples_num=1;
    for(int p=0;p<samples_num;p++)
    {

      bool points_ok = false;
      CvPoint2D32f *p1,*p2;

      while(!points_ok)
      {
        int s1=0;
        int s2=0;
        while(s1==s2)
        {
          s1 = rand() % all_points_seq->total;
          s2 = rand() % all_points_seq->total;
        //std::cout << "\ns1: " << s1 << " s2: " << s2 << "\n";

        }

        p1 = CV_GET_SEQ_ELEM( CvPoint2D32f, all_points_seq, s1 );

        //std::cout << "\np1.x: " << p1->x << " p1.y: " << p1->y << "\n";

        if(p1->x<kernel_size/2 || p1->x>(roi_size - kernel_size/2) || 
          p1->y<kernel_size/2 || p1->y>(roi_size - kernel_size/2)) continue;

        p2 = CV_GET_SEQ_ELEM( CvPoint2D32f, all_points_seq, s2 );

        if(p2->x<kernel_size/2 || p2->x>(roi_size - kernel_size/2) || 
          p2->y<kernel_size/2 || p2->y>(roi_size - kernel_size/2)) continue;

        //std::cout << "\np2.x: " << p2->x << " p2.y: " << p2->y << "\n";

        double a = p2->x - p1->x;
        double b = p2->y - p1->y;

        double dist = sqrt(a*a + b*b);

        if(dist < min_dist || dist > max_dist) continue;

        points_ok = true;
        //std::cout << "\na: " << a << " b: " << b << "\n";

      }

      float px,py;
      MyLine l1,l2;
      //float *l1 = new float[4];
      //float *l2 = new float[4];
      MyLine* line;

      for(int c=0;c<2;c++)
      {
        if(c==0) { 
          px = p1->x; 
          py = p1->y;
          line = &l1;
        }
        else { 
          px = p2->x; 
          py = p2->y;
          line = &l2;
        }



        //std::cout << "\nx: " << px << " y: " << py << "\n";

        IplImage* kernel = cvCreateImage( cvSize(kernel_size,kernel_size), 8, 1 );
        cvSetImageROI(roi,cvRect(px-kernel_size/2,py-kernel_size/2,kernel_size,kernel_size)); 
        cvCopy(roi,kernel); 
        cvResetImageROI(roi);

        CvMemStorage* storage = cvCreateMemStorage(0);

        CvSeq* point_seq = cvCreateSeq( CV_32FC2, sizeof(CvSeq), sizeof(CvPoint2D32f), storage );

        int nl= kernel->height; // number of lines
        int nc= kernel->width;// total number of element per line
        int step= kernel->widthStep; // effective width

        unsigned char *data= reinterpret_cast<unsigned char *>(kernel->imageData);

        int num_points_found = 0;
        for (int i=0; i<nl; i++) {
          for (int j=0; j<nc; j++) {
            if(data[j]>150)
            {
              num_points_found++;
              cvSeqPush(point_seq, &cvPoint2D32f(px+j-kernel_size/2,py+i-kernel_size/2));
              //std::cout << "[j,i]: " << j << " " << i << " px+j-kernel_size/2: " << px+j-kernel_size/2 << " py+i-kernel_size/2: " << py+i-kernel_size/2 << "\n";
            }
            //else std::cout << "missing\n";
          }
          data+= step;  // next line
        }

        if(num_points_found>10)
        {
          float *fline = new float[4];
          cvFitLine(point_seq,CV_DIST_L2,0,0.001,0.001,fline);

          line->vx = fline[0];
          line->vy = fline[1];
          line->px = fline[2];
          line->py = fline[3];
          //std::cout << "line params: " << line[0] << ":" << line[1] << ":" << line[2] << ":" << line[3] << "\n";

          //cvSeqPush(lines, &myLine(line[0], line[1], line[2], line[3]));

          //CvPoint pt1, pt2;

          //pt1.x = line[0]*100 + line[2];
          //pt1.y = line[1]*100 + line[3];
          //pt2.x = line[2];
          //pt2.y = line[3];
          //cvLine( color_roi, pt1, pt2, CV_RGB(255,0,0), 1, 8 );
        }
        //std::stringstream filename;
        //filename << argv[2] << p << ".jpg";
        //std::cout << filename.str().c_str() << "\n\n";
        //cvSaveImage( filename.str().c_str(), kernel );

      }

      CvPoint m, t;
      m.x = cvRound(0.5*(l1.px + l2.px));
      m.y = cvRound(0.5*(l1.py + l2.py));

      double num = l1.vx*(l2.py-l1.py) + l1.vy*(l1.px-l2.px);
      double denom = l1.vy*l2.vx - l1.vx*l2.vy;
      double mu = num/denom;

      t.x = cvRound(l2.vx * mu + l2.px);
      t.y = cvRound(l2.vy * mu + l2.py);

      CvPoint e, tm;

      tm.x = m.x - t.x;
      tm.y = m.y - t.y;

      //double e_length = roi_size*roi_size*sqrt(2.0)/sqrt((float)(tm.x*tm.x + tm.y*tm.y));
      double e_length = ray_vote/sqrt((float)(tm.x*tm.x + tm.y*tm.y));
      e.x = m.x + tm.x*e_length;
      e.y = m.y + tm.y*e_length;

      //cvCircle( color_roi, m, 1, CV_RGB(0,255,0), -1, 8, 0 );
      //cvCircle( color_roi, t, 2, CV_RGB(0,0,255), -1, 8, 0 );
      //cvLine( color_roi, cvPoint(l1.px,l1.py), cvPoint(l2.px,l2.py), CV_RGB(128,0,0), 1, 8 );
      //cvLine( color_roi, cvPoint(l1.px,l1.py), t, CV_RGB(255,0,0), 1, 8 );
      //cvLine( color_roi, cvPoint(l2.px,l2.py), t, CV_RGB(255,0,0), 1, 8 );

      if(cvClipLine( cvSize(roi->width,roi->height), &m, &e ))
      {

        //cvLine( color_roi, m, e, CV_RGB(0,255,0), 1, 8 );

        //std::cout << " ";// << m.x << " " << m.y << " " << e.x << " " << e.y << "i ";

        CvLineIterator iterator;
        int count = cvInitLineIterator( roi, m, e, &iterator, 8, 0 );

        for( int i = 0; i < count; i++ )
        {
          int offset, x, y;
          /* assume that ROI is not set, otherwise need to take it into account. */
          offset = iterator.ptr - (uchar*)(roi->imageData);
          y = offset/(roi->widthStep);
          x = offset%(roi->widthStep);
          //x = (offset - y*roi->widthStep);// /(3*sizeof(uchar) /* size of pixel */);

          //For better visibility in accululator; should be incremented only by one
          //accum_data[y*accum->cols+x]+= 10;
          accum_data[y*accum->cols+x]++;
          CV_NEXT_LINE_POINT(iterator);

          //printf("(%d,%d, %d)\n", x, y, accum_data[y*accum->cols+x] );
        }
      }

    }

    double min_val=0, max_val=0;
    CvPoint min_loc, max_loc;
    cvMinMaxLoc( accum_image, &min_val, &max_val, &min_loc, &max_loc );

    std::cout << "(" << max_loc.x << ", " << max_loc.y  << ", " << max_val << ")\n";

    //if(max_val > 0)
    //{
    //  std::stringstream maxtext;
    //  maxtext << max_val;
    //  CvFont font1;
    //  cvInitFont( &font1, CV_FONT_VECTOR0, 0.2f, 0.2f, 0.0f,0.5 );
    //  cvPutText(color_roi, maxtext.str().c_str() , max_loc, &font1, CV_RGB(0,255,255)); 
    //}

    std::stringstream filename;
    filename << argv[2] << "accum.jpg";
    //std::cout << filename.str().c_str() << " :uchar: " << 3*sizeof(uchar) << ":depth:" << accum_image->depth << "\n\n";
    cvSaveImage( filename.str().c_str(), accum_image );

    while(max_val > 30)
    {
      std::stringstream maxtext;
      maxtext << max_val;
      CvFont font1;
      cvInitFont( &font1, CV_FONT_VECTOR0, 0.8f, 0.8f, 0.0f, 2 );
      cvPutText(color_roi, maxtext.str().c_str() , max_loc, &font1, CV_RGB(0,255,255)); 

      cvCircle(accum_image,max_loc,ray_vote,CV_RGB(0,0,0), -1, 8, 0 );
      cvMinMaxLoc( accum_image, &min_val, &max_val, &min_loc, &max_loc );

    }

    //cvCircle( color_roi, max_loc, 3, CV_RGB(0,255,255), -1, 8, 0 );

    //cvClearSeq( lines );
    //contour = contour->h_next;   
  //} 

  //std::cout << "hier";
  cvSaveImage( argv[2], color_roi );

  //if(img) cvReleaseImage(&img);
  //if(color_dst) cvReleaseImage(&color_dst);


}
}
