/******************************************************************************
*
*   This file is part of the software tool "Cervix Image Processing 2 (CIP2)".
*
*   Author: Christoph Bergmeir
*   Supervisor: José M. Benítez
*   Copyright (c) DiCITS Lab, Sci2s group, DECSAI, University of Granada.
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
*   MA  02110-1301, USA.
*
******************************************************************************/

//#include "itkThresholdImageFilter.h"
#include "itkDoubleThresholdImageFilter.h"

#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"

int main( int argc, char * argv[] )
{

  if( argc < 7 )
    {
    std::cerr << "Usage: " << argv[0] << " inputImageFile ";
    std::cerr << " outputImageFile thres1 thres2 thres3 thres4" << std::endl;  
    return EXIT_FAILURE;
    }

  int thres1 = atoi( argv[3] );
  int thres2 = atoi( argv[4] );
  int thres3 = atoi( argv[5] );
  int thres4 = atoi( argv[6] );

  //std::string in_filename = "1-2.jpg";
  //std::string out_filename1 = "1-2_doublethres1.jpg";
  //std::string out_filename2 = "1-2_processed2.jpg";
  //std::string out_filename3 = "1-2_processed3.jpg";
  
  typedef  unsigned char  PixelType;
  typedef itk::Image< PixelType,  2 >   ImageType;
  typedef itk::ImageFileReader< ImageType >  ReaderType;
  typedef itk::ImageFileWriter< ImageType >  WriterType;

  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName( argv[1] );

  typedef itk::DoubleThresholdImageFilter< ImageType, ImageType >  FilterType;
  FilterType::Pointer filter = FilterType::New();
  
  filter->SetInput( reader->GetOutput() );
  filter->SetOutsideValue( 0 );
  filter->SetThreshold1( thres1 );
  filter->SetThreshold2( thres2 );
  filter->SetThreshold3( thres3 );
  filter->SetThreshold4( thres4 );
  filter->Update();

  WriterType::Pointer writer = WriterType::New();
  writer->SetInput( filter->GetOutput() );

  writer->SetFileName( argv[2] );
  writer->Update();


  return EXIT_SUCCESS;
}

