/******************************************************************************
*
*   This file is part of the software tool "Cervix Image Processing 2 (CIP2)".
*
*   Author: Christoph Bergmeir
*   Supervisor: José M. Benítez
*   Copyright (c) DiCITS Lab, Sci2s group, DECSAI, University of Granada.
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
*   MA  02110-1301, USA.
*
******************************************************************************/

#include "itkOtsuMultipleThresholdsCalculator.h"

#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkScalarImageToHistogramGenerator.h"
#include "itkBinaryThresholdImageFilter.h"
#include "itkNumericTraits.h"

#include <stdio.h>
#include <iostream>
#include <math.h>

void convertXYtoLatLng(int x, int y, double& lat, double& lng)
{
////Not needed at the moment, and the computation is faulty..
//  double o1x = 0.703186033;
//  double o2x = 65594.306219867;
//
//  //lat=73.986237
//  //sin=0.961195457
//  //arg_log=50.5403576
//  //ln=3.922772178
//  //x=0.624328583
//  double o1y = 40911.5;
//  double o2y = 65528.795478211;
//
//  //double o1y = 46124.999978136;
//  //double o2y = 65594.306219867;
//
//  lng = (((double) x) / o2x - o1x) * 180;
//  lat = (M_PI / 2 - 2* atan (exp((y - o1y) * M_PI / o2y))) * 180 / M_PI;

  lat = 0;
  lng = 0;
}

int main(int argc, char * argv[])
{
  typedef unsigned short InputPixelType;
  typedef unsigned char OutputPixelType;

  typedef itk::Image<InputPixelType, 2> InputImageType;
  typedef itk::Image<OutputPixelType, 2> OutputImageType;

  typedef itk::Statistics::ScalarImageToHistogramGenerator<InputImageType>
      ScalarImageToHistogramGeneratorType;

  typedef ScalarImageToHistogramGeneratorType::HistogramType HistogramType;

  typedef itk::OtsuMultipleThresholdsCalculator<HistogramType> CalculatorType;

  typedef itk::ImageFileReader<InputImageType> ReaderType;
  typedef itk::ImageFileWriter<OutputImageType> WriterType;

  typedef itk::BinaryThresholdImageFilter<InputImageType, OutputImageType>
      FilterType;

  double minPercentage = atof(argv[4]);
  double maxPercentage = atof(argv[5]);

  std::ofstream outTextFile(argv[3]);

  ScalarImageToHistogramGeneratorType::Pointer scalarImageToHistogramGenerator =
      ScalarImageToHistogramGeneratorType::New();

  CalculatorType::Pointer calculator = CalculatorType::New();
  FilterType::Pointer filter = FilterType::New();

  ReaderType::Pointer reader = ReaderType::New();
  WriterType::Pointer writer = WriterType::New();

  scalarImageToHistogramGenerator->SetNumberOfBins(128);
  calculator->SetNumberOfThresholds(2);

  const OutputPixelType outsideValue = 0;
  const OutputPixelType insideValue = 255;

  filter->SetOutsideValue(outsideValue);
  filter->SetInsideValue(insideValue);

  reader->SetFileName(argv[1]);

  scalarImageToHistogramGenerator->SetInput(reader->GetOutput());
  calculator->SetInputHistogram(scalarImageToHistogramGenerator->GetOutput());
  filter->SetInput(reader->GetOutput());

  //Invoke pipeline
  try
  {
    reader->Update();
  } catch (itk::ExceptionObject & excp)
  {
    std::cerr << "Exception thrown while reading image" << excp << std::endl;
  }
  scalarImageToHistogramGenerator->Compute();

  try
  {
    calculator->Update();
  } catch (itk::ExceptionObject & excp)
  {
    std::cerr << "Exception thrown " << excp << std::endl;
  }

  const CalculatorType::OutputType &thresholdVector = calculator->GetOutput();
  CalculatorType::OutputType::const_iterator itNum = thresholdVector.begin();

  InputPixelType lowerThreshold = 0;
  InputPixelType upperThreshold = thresholdVector.at(1);
  std::cout << "Threshold: " << upperThreshold << "\n";

  std::string outputFile(argv[2]);

  std::stringstream otsuOutfile;

  otsuOutfile << outputFile << ".otsu.png";

  filter->SetLowerThreshold(lowerThreshold);
  filter->SetUpperThreshold(upperThreshold);

  writer->SetInput(filter->GetOutput());
  writer->SetFileName(otsuOutfile.str());

  try
  {
    writer->Update();
  } catch (itk::ExceptionObject & excp)
  {
    std::cerr << "Exception thrown " << excp << std::endl;
  }

  OutputImageType::Pointer image = OutputImageType::New();

  filter->Update();
  image = filter->GetOutput();

  int roiSizeX = 18;
  int roiSizeY = 18;

  int fullRoiSizeX = 1920;
  int fullRoiSizeY = 1920;

  int imageSizeX = image->GetLargestPossibleRegion().GetSize(0);
  int imageSizeY = image->GetLargestPossibleRegion().GetSize(1);

  int amountRoisX = imageSizeX / roiSizeX;
  int amountRoisY = imageSizeY / roiSizeY;

  std::cout << "imageSize: " << imageSizeX << " : " << imageSizeY << "\n";
  std::cout << "amountRois: " << amountRoisX << " : " << amountRoisY << "\n";

  OutputImageType::Pointer outputImage = OutputImageType::New();
  itk::ImageRegion<2> outputRegion;
  itk::ImageRegion<2> requestedRegion;

  outputRegion.SetIndex(0, 0);
  outputRegion.SetIndex(1, 0);
  outputRegion.SetSize(0, amountRoisX);
  outputRegion.SetSize(1, amountRoisY);

  outputImage->SetRequestedRegion(outputRegion);
  outputImage->SetBufferedRegion(outputRegion);
  outputImage->SetLargestPossibleRegion(outputRegion);
  outputImage->Allocate();

  itk::ImageRegionIterator<OutputImageType> outputIt(outputImage, outputRegion);
  outputIt.GoToBegin();

  int amountInterestingRois = 0;

  for (int y = 0; y < amountRoisY; y++)
  {
    for (int x = 0; x < amountRoisX; x++)
    {

      requestedRegion.SetIndex(0, x * roiSizeX);
      requestedRegion.SetIndex(1, y * roiSizeY);
      requestedRegion.SetSize(0, roiSizeX);
      requestedRegion.SetSize(1, roiSizeY);

      //image->SetRequestedRegion( requestedRegion );
      //image->SetBufferedRegion( requestedRegion );
      //image->SetLargestPossibleRegion( requestedRegion );
      //image->Allocate();

      itk::ImageRegionIterator<OutputImageType> imageIt(image, requestedRegion);

      int sum = 0;
      imageIt.GoToBegin();
      while (!imageIt.IsAtEnd())
      {
        OutputPixelType pixel = imageIt.Get();
        if (pixel != 0)
          sum++;
        ++imageIt;
      }

      double percentage = ((double) sum) / (roiSizeX * roiSizeY);

      //std::cout << "Pixels marked in the image: " << sum << " : " << percentage << "\n";

      OutputPixelType outPixel = 0;
      if (percentage > minPercentage && percentage < maxPercentage)
      {
        //int regionCenterX = x * roiSizeX + roiSizeX / 2;
        //int regionCenterY = y * roiSizeY + roiSizeY / 2;

        int regionCenterX = x * fullRoiSizeX + fullRoiSizeX / 2;
        int regionCenterY = y * fullRoiSizeY + fullRoiSizeY / 2;

        std::cout << "Interesting region " << amountInterestingRois << "at: "
            << regionCenterX << " : " << regionCenterY << "\n";

        double lat, lng;
        convertXYtoLatLng(regionCenterX, regionCenterY, lat, lng);

        outTextFile << amountInterestingRois << " " << regionCenterX << " "
            << regionCenterY << " " << fullRoiSizeX << " "
            << fullRoiSizeY << " "<< lat << " " << lng << "\n";
        outPixel = 255;
        amountInterestingRois++;
      }
      //outPixel = (OutputPixelType) percentage * 100;
      outputIt.Set(outPixel);
      ++outputIt;

    }
  }

  writer->SetInput(outputImage);
  writer->SetFileName(outputFile);

  try
  {
    writer->Update();
  } catch (itk::ExceptionObject & excp)
  {
    std::cerr << "Exception thrown " << excp << std::endl;
  }

  return EXIT_SUCCESS;
}

