/******************************************************************************
*
*   This file is part of the software tool "Cervix Image Processing 2 (CIP2)".
*
*   Author: Christoph Bergmeir
*   Supervisor: José M. Benítez
*   Copyright (c) DiCITS Lab, Sci2s group, DECSAI, University of Granada.
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
*   MA  02110-1301, USA.
*
******************************************************************************/

#include <vips/vips>

//#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/path.hpp"
//#include "boost/progress.hpp"
//#include "boost/regex.hpp"

#include <iostream>
#include <fstream>
#include <string>
//#include <list>


namespace fs = boost::filesystem;

using namespace vips;

void stringExplode(std::string str, std::string separator, std::vector<std::string>* results)
{
  //found has to be a signed int
  int found;
  found = str.find_first_of(separator);
  while (found != std::string::npos)
  {
    if (found > 0)
      results->push_back(str.substr(0, found));

    str = str.substr(found + 1);
    found = str.find_first_of(separator);
  }
  if (str.length() > 0)
  {
    results->push_back(str);
  }
}

int main (int argc, char **argv)
{
  std::string inFilename(argv[1]);
  //std::string inFilenameMarkers(argv[2]);
  std::ifstream inFileMarkers(argv[2]);
  std::string outDir(argv[3]);

  std::vector<int> markersId, markersX, markersY, markersSizeX, markersSizeY;

  std::string outDirEnding = outDir.substr(outDir.length()-1);
  if(outDirEnding.compare("/")) outDir.append("/");

  fs::path inFilenamePath( inFilename );

  std::string fn = inFilenamePath.filename().string();

  std::string filenameNoEnding = fn.substr(0, fn.length()-4);
  std::cout << filenameNoEnding << "\n";

//Open the large image and get its size
  VImage inImage (inFilename.c_str());

  int imgSizeX = inImage.Xsize();
  int imgSizeY = inImage.Ysize();

  std::cout << "Image dimensions: " << imgSizeX << " : " << imgSizeY << "\n";




//Process marker txt file
  std::string line;

  if (inFileMarkers.is_open())
  {

    while (!inFileMarkers.eof())
    {
      std::getline(inFileMarkers, line);
      //std::cout << line << std::endl;

      std::vector<std::string> parameters;
      stringExplode(line, " ", &parameters);

      if (parameters.size() < 5) continue;

      int id = atoi(parameters[0].c_str());
      int x  = atoi(parameters[1].c_str());
      int y  = atoi(parameters[2].c_str());
      int sizeX  = atoi(parameters[3].c_str());
      int sizeY  = atoi(parameters[4].c_str());

      //std::cout << "x,y: " << x << " : " << y << "\n";

      //perform some validity checks on the markers (otherwise extract_area will fail)
      if (sizeX == 0 || sizeY == 0) continue;

      x = x-sizeX/2;
      y = y-sizeY/2;

      if (x < 0 || y < 0 || x+sizeX > imgSizeX || y+sizeY > imgSizeY) continue;

      markersId.push_back(id);
      markersX.push_back(x);
      markersY.push_back(y);
      markersSizeX.push_back(sizeX);
      markersSizeY.push_back(sizeY);

    }
    inFileMarkers.close();
  }
  else
  {
    std::cout << "Unable to open marker file";
    return 1;
  }

  //------------------------

  std::cout << "length of input vectors: " << markersId.size() << " : " << markersX.size() << " : " << markersY.size() << "\n";




    for(unsigned int i=0;i < markersX.size(); i++)
    {
      std::stringstream outFilename;
      outFilename << outDir << filenameNoEnding << "_" << markersX[i] << "_" << markersY[i]
                  << "_" << markersSizeX[i] << "_" << markersSizeY[i] << ".jpg";

      std::cout << outFilename.str() << "\n";

      VImage region = inImage.extract_area( markersX[i], markersY[i], markersSizeX[i], markersSizeY[i] );


      region.write(outFilename.str().c_str());

    };

    return (0);
}
