/******************************************************************************
*
*   This file is part of the software tool "Cervix Image Processing 2 (CIP2)".
*
*   Author: Christoph Bergmeir
*   Supervisor: José M. Benítez
*   Copyright (c) DiCITS Lab, Sci2s group, DECSAI, University of Granada.
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
*   MA  02110-1301, USA.
*
******************************************************************************/

#include <vips/vips>

//#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/path.hpp"
//#include "boost/progress.hpp"
//#include "boost/regex.hpp"

#include <iostream>
//#include <list>


namespace fs = boost::filesystem;

using namespace vips;


int main (int argc, char **argv)
{
    std::string inFilename(argv[1]);
    std::string outDir(argv[2]);

    std::string outDirEnding = outDir.substr(outDir.length()-1);
    if(outDirEnding.compare("/")) outDir.append("/");

	VImage inImage (inFilename.c_str());
	//fs::path outDir( argv[2]);

    int imgSizeX = inImage.Xsize();
    int imgSizeY = inImage.Ysize();

    std::cout << "Image dimensions: " << imgSizeX << " : " << imgSizeY << "\n";

    int roiSizeX = 1950;
    int roiSizeY = 1950;

    int roiOverlapX = 150;
    int roiOverlapY = 150;

    int numRoisX = imgSizeX / (roiSizeX - roiOverlapX);
    int numRoisY = imgSizeY / (roiSizeY - roiOverlapY);

    int startX = 0;
    int startY = 0;

    int y = startY;

    fs::path inFilenamePath( inFilename );

    std::string filenameNoEnding = inFilenamePath.leaf().string().substr(0, inFilenamePath.leaf().string().length()-4);
    std::cout << filenameNoEnding << "\n";

    for(int cy = 0; cy < numRoisY; cy++)
    {
      int x = startX;
      for(int cx = 0; cx < numRoisX; cx++)
      {
    	  std::stringstream outFilename;
    	  VImage region = inImage.extract_area( x, y, roiSizeX, roiSizeY );

    	  outFilename << outDir << filenameNoEnding << "_" << cx << "_" << cy << "_" << x << "_" << y << ".jpg";
    	  std::cout << outFilename.str() << "\n";
    	  region.write(outFilename.str().c_str());

    	  x += (roiSizeX-roiOverlapX);
      }
      y += (roiSizeY-roiOverlapY);
    }

    return (0);
}
