/******************************************************************************
*
*   This file is part of the software tool "Cervix Image Processing 2 (CIP2)".
*
*   Author: Christoph Bergmeir
*   Supervisor: José M. Benítez
*   Copyright (c) DiCITS Lab, Sci2s group, DECSAI, University of Granada.
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
*   MA  02110-1301, USA.
*
******************************************************************************/

#include <vips/vips>

#include "boost/filesystem.hpp"
#include "boost/progress.hpp"
#include "boost/regex.hpp"

#include <iostream>
#include <list>

namespace fs = boost::filesystem;

using namespace vips;

boost::regex e("\\D", boost::regbase::normal | boost::regbase::icase);

unsigned tokenise(std::list<std::string>& l, std::string& s)
{
  return boost::regex_split(std::back_inserter(l), s, e);
}

unsigned extractCoordinatesFromFilename(int& x, int& y, std::string& inString, bool old_format)
{
  std::string s(inString);

  std::list<std::string> l;

  tokenise(l, s);

  while (!l.empty())
  {
    s = l.back();
    if (s != "")
    {
      if(old_format)
      {
        y = atoi(s.c_str());
        l.pop_back();
        s = l.back();
        x = atoi(s.c_str());
      }
      else
      {
//        size_y = atoi(l.pop_back().c_str());
//        size_x = atoi(l.pop_back().c_str());
        l.pop_back();
        l.pop_back();
        s = l.back();
        y = atoi(s.c_str());
        l.pop_back();
        s = l.back();
        x = atoi(s.c_str());
      }
      return 0;
    }
    l.pop_back();
  }
  return 1;
}

int main(int argc, char **argv)
{
  std::string inBigFile(argv[1]);
  std::string outDebugFile(argv[2]);
  std::string inPathInsertFiles(argv[3]);
  std::string outDir(argv[4]);

  int shrinkTiles = atoi(argv[5]);
  int retile = atoi(argv[6]);
  int oldNamingFormatInt = atoi(argv[7]);

  bool oldNamingFormat = false;
  if(oldNamingFormatInt == 1) oldNamingFormat = true;

  std::string outDirEnding = outDir.substr(outDir.length() - 1);
  if (outDirEnding.compare("/"))
    outDir.append("/");

  //boost::progress_timer t( std::clog );

  fs::path inPath(inPathInsertFiles);
  //	  fs::path inPath( fs::initial_path<fs::path>() );


  /*	  if ( argc > 1 )
   inPath = fs::system_complete( fs::path( argv[1] ) );
   else
   std::cout << "\nusage:   simple_ls [path]" << std::endl;*/

  unsigned long file_count = 0;

  std::list<std::string> inFiles;

  if (!fs::exists(inPath))
  {
    std::cout << "\nNot found: " << inPath.filename().string()
        << " no processed tiles will be used" << std::endl;
    //return 1;
  }

  if (fs::is_directory(inPath))
  {
    std::cout << "\nIn directory: " << inPath.parent_path().string() << "\n\n";
    fs::directory_iterator end_iter;
    for (fs::directory_iterator dir_itr(inPath); dir_itr != end_iter; ++dir_itr)
    {
      try
      {
        //for compatibility on dicits, use the old, deprecated name
        //if (fs::is_regular_file(dir_itr->status()))
        if (fs::is_regular(dir_itr->status()))
        {
          ++file_count;
          //	          std::cout << dir_itr->path().filename() << "\n";
          //            std::string currentFile(dir_itr->path().filename());
          std::cout << dir_itr->path().string() << "\n"; //native_file_string
          std::string currentFile(dir_itr->path().string()); //native_file_string
          std::string fileEnding = currentFile.substr(currentFile.length() - 4);
          if (!fileEnding.compare(".jpg") || !fileEnding.compare(".png"))
            inFiles.push_back(currentFile);

        }
      } catch (const std::exception & ex)
      {
        //doesn't work on dicits:
        //std::cout << dir_itr->path().filename() << " " << ex.what() << std::endl;
        std::cout << ex.what() << std::endl;
      }
    }
  }
  /*	  else // must be a file
   {
   std::cout << "\nFound: " << inPath.filename().string() << "\n";
   }*/

  VImage bigImage(inBigFile.c_str());

  std::vector<int> regionX, regionY, regionSizeX, regionSizeY;

  while (inFiles.size())
  {
    int x = 0, y = 0;
    std::string currentFile = *(inFiles.begin());
    inFiles.pop_front();
    extractCoordinatesFromFilename(x, y, currentFile, oldNamingFormat);
    std::cout << x << " : " << y << " Coordinates extracted from file: " << currentFile << std::endl;

std::cout << currentFile.c_str() << " ..\n";

    VImage smallImage = VImage(currentFile.c_str());

    std::cout << "Image opened..\n";

    if (shrinkTiles != 0)
    {
      smallImage = smallImage.shrink(2, 2);
      x = x / 2;
      y = y / 2;
      std::cout << "Edited coordinates: " << x << " : " << y << "\n";
    };

    int smallImageSizeX = smallImage.Xsize();
    int smallImageSizeY = smallImage.Ysize();

    regionX.push_back(x);
    regionY.push_back(y);
    regionSizeX.push_back(smallImageSizeX);
    regionSizeY.push_back(smallImageSizeY);

    bigImage = bigImage.insert(smallImage, x, y);
  }

  //std::cout << "Shrinking image for debug purposes..\n";
  //VImage shrinkedBigImage = bigImage.shrink(4, 4);
  //std::cout << "Saving shrinked image as jpg..\n";
  //shrinkedBigImage.write(outDebugFile.c_str());

  //TODO: generate dir, if it doesn't exist
  //$progOutput = shell_exec(str_replace($separator_not, $separator, "mkdir ./tiles"));
  //$progOutput = shell_exec(str_replace($separator_not, $separator, "mkdir ./tiles/$outDir"));

  int imgSizeX = bigImage.Xsize();
  int imgSizeY = bigImage.Ysize();

  int sourceTilesX = imgSizeX / 256;
  int sourceTilesY = imgSizeY / 256;

  int maxZoom = log2(sourceTilesX) + 1;
  int maxEdgeTile = pow(2, maxZoom);
  int maxEdgePix = maxEdgeTile * 256;
  int xPad = (maxEdgePix - imgSizeX) / 2;
  int yPad = (maxEdgePix - imgSizeY) / 2;

  int xOffset = (maxEdgeTile - sourceTilesX) / 2;
  int yOffset = (maxEdgeTile - sourceTilesY) / 2;

  std::cout << "Generating " << maxEdgePix << " by " << maxEdgePix
      << "square image...\n";

  bigImage = bigImage.embed(0, xPad, yPad, maxEdgePix, maxEdgePix);

  int rTileXPad = xPad;
  int rTileYPad = yPad;

  //std::cout << "Shrinking image for debug purposes..\n";
  //VImage shrinkedBigImage2 = bigImage.shrink(4, 4);
  //std::cout << "Saving shrinked image as jpg..\n";
  //shrinkedBigImage2.write(outDebugFile.c_str());

  //VImage tmpImage;

  for (int zoom = maxZoom; zoom >= 0; zoom--)
  {
    int tiles = pow(2, zoom);
    std::cout << "Zoom: " << zoom << " Tiles: " << tiles << "\n";
    if (zoom < maxZoom)
      bigImage = bigImage.shrink(2, 2);

    //----------------------------------------------
    if(zoom == 6 || zoom == 3)
    {
      std::cout << "generating copy in memory of shrinked image (for performance purposes)..\n";
      bigImage = bigImage.write();
//      bigImage.write("tmp.v");
//      VImage tmpImage("tmp.v");
//      bigImage = tmpImage;
//      //im_close(bigImage);
//      //bigImage = im_open(outDebugFile.c_str());
      std::cout << "finished generating copy in memory of shrinked image..\n";
    }
    //----------------------------------------------

    if(retile == 0)
    {
      for (int x = xOffset; x < tiles - xOffset; x++)
      {
        for (int y = yOffset; y < tiles - yOffset; y++)
        {
          int left = x * 256;
          int top = y * 256;

          std::stringstream outFilename;
          VImage region = bigImage.extract_area(left, top, 256, 256);
          outFilename << outDir << "tile_" << zoom << "_" << x << "_" << y
              << ".jpg";
          region.write(outFilename.str().c_str());
        };
      };
    }
    else
    {
      for(int regions = 0; regions<regionX.size(); regions++)
      {
        int rCoordX = regionX[regions] + rTileXPad;
        int rCoordY = regionY[regions] + rTileYPad;

        int rTileCoordX = rCoordX / 256;
        int rTileCoordY = rCoordY / 256;

        int rTileAmountX = (regionSizeX[regions] / 256) + 1;
        int rTileAmountY = (regionSizeY[regions] / 256) + 1;

        if(zoom <= 1)
        {
          //TODO: This should be handled nicer...seems to be a rounding problem
          //Furthermore are the tiles generated for every roi. But there are always only 4, respectively 1 tiles present

//          std::cout << "rTileAmountX: " << rTileAmountX << " rTileAmountY: " << rTileAmountX << " regionSizeX[regions]: " << regionSizeX[regions] << "\n";
//          std::cout << "regionSizeY[regions]: " << regionSizeY[regions] << " rTileCoordX: " << rTileCoordX << " rTileCoordY: " << rTileCoordY << "\n";
//          std::cout << "rCoordX: " << rCoordX << " rCoordY: " << rCoordY << " rTileXPad: " << rTileXPad << " rTileYPad: " << rTileYPad << "\n\n";
          rTileAmountX--;
          rTileAmountY--;
        }
        //-----------------
        for (int x = rTileCoordX; x <= rTileCoordX + rTileAmountX; x++)
        {
          for (int y = rTileCoordY; y <= rTileCoordY + rTileAmountY; y++)
          {
            int left = x * 256;
            int top = y * 256;

            std::stringstream outFilename;
            outFilename << outDir << "tile_" << zoom << "_" << x << "_" << y
                << ".jpg";

            try
            {
              VImage region = bigImage.extract_area(left, top, 256, 256);
              region.write(outFilename.str().c_str());
            }
            catch(...)
            {
              std::cout << "File: " << outFilename.str() << " : " << "_" << left << "_" << top << "\n";
              exit(1);

            }
          };
        };
        //------------------------

        regionX[regions] = regionX[regions] / 2;
        regionY[regions] = regionY[regions] / 2;
        regionSizeX[regions] = regionSizeX[regions] / 2;
        regionSizeY[regions] = regionSizeY[regions] / 2;

      }
      rTileXPad = rTileXPad / 2;
      rTileYPad = rTileYPad / 2;
    }

    maxEdgeTile = maxEdgeTile / 2;
    xOffset = xOffset / 2;
    yOffset = yOffset / 2;
    if (xOffset == 1)
      xOffset = 0;
    if (yOffset == 1)
      yOffset = 0;
  }

  /*try {

   VImage jim = (fred * 1.2).conv (sharp);

   // write to file argv[2]
   //jim.write (argv[2]);
   }
   catch (VError &e) {
   e.perror (argv[0]);
   }*/

  return (0);
}
