
//NYU School of Medicine GMap Microscopy Functions - NYUSM Division of Educational Informatics
//Released under the MIT Educational License
//Portions of Javascript from http://econym.googlepages.com/
//Portions of Javascript from http://gmaps-utility-library.googlecode.com/svn/trunk/labeledmarker

//The URL to the directory containing tiles for a given image

//var baseimageurl = 'http://nemo5.ugr.es/virtualmicroscope/6671_jpeg/tiles';

//var baseimageurl = 'file://localhost/home/bergmeir/Software/virtualmicroscope/virtualmicroscope-read-only/out/test';
//var baseimageurl = 'http://informatics.med.nyu.edu/microscopy/images/demoslide';
//the Max zoom level for the image corresponding to the highest resolution tile layer
//This changes from slide to slide and for the sake of this example is hard coded but can be detected dynamically.


//-----------------------------------------
      // ====== Create the Euclidean Projection for the flat map ======
      // == Constructor ==
      function EuclideanProjection(a){
        this.pixelsPerLonDegree=[];
        this.pixelsPerLonRadian=[];
        this.pixelOrigo=[];
        this.tileBounds=[];
        var b=256;
        var c=1;
        for(var d=0;d<a;d++){
          var e=b/2;
          this.pixelsPerLonDegree.push(b/360);
          this.pixelsPerLonRadian.push(b/(2*Math.PI));
          this.pixelOrigo.push(new GPoint(e,e));
          this.tileBounds.push(c);
          b*=2;
          c*=2
        }
      }
 
      // == Attach it to the GProjection() class ==
      EuclideanProjection.prototype=new GProjection();
 
 
      // == A method for converting latitudes and longitudes to pixel coordinates == 
      EuclideanProjection.prototype.fromLatLngToPixel=function(a,b){
        var c=Math.round(this.pixelOrigo[b].x+a.lng()*this.pixelsPerLonDegree[b]);
        var d=Math.round(this.pixelOrigo[b].y+(-2*a.lat())*this.pixelsPerLonDegree[b]);
        return new GPoint(c,d)
      };

      // == a method for converting pixel coordinates to latitudes and longitudes ==
      EuclideanProjection.prototype.fromPixelToLatLng=function(a,b,c){
        var d=(a.x-this.pixelOrigo[b].x)/this.pixelsPerLonDegree[b];
        var e=-0.5*(a.y-this.pixelOrigo[b].y)/this.pixelsPerLonDegree[b];
        return new GLatLng(e,d,c)
      };

      // == a method that checks if the y value is in range, and wraps the x value ==
      EuclideanProjection.prototype.tileCheckRange=function(a,b,c){
        var d=this.tileBounds[b];
        if (a.y<0||a.y>=d) {
          return false;
        }
        if(a.x<0||a.x>=d){
          a.x=a.x%d;
          if(a.x<0){
            a.x+=d;
          }
        }
        return true
      }

      // == a method that returns the width of the tilespace ==      
      EuclideanProjection.prototype.getWrapWidth=function(zoom) {
        return this.tileBounds[zoom]*256;
      }
//------------------------------------------

var maxZoomLevel = 9;

var map;
var ovcontrol;
var ovmap;
var projection;

var currentMarker;

//the default view upon opening
var centerLat = 0, centerLong = 0, initialZoom = 2;

window.onresize = handleResize;
window.onload = init;
window.onunload = GUnload;

var showToogleText = Array (false, false, false);

//MAP UTILITY FUNCTIONS
function windowHeight() {
  // Standard browsers (Mozilla, Safari, etc.)
  if (self.innerHeight) {
    return self.innerHeight;
  }
  // IE 6
  if (document.documentElement && document.documentElement.clientHeight) {
   return document.documentElement.clientHeight;
  }
  // IE 5
  if (document.body) {
    return document.body.clientHeight;
  }
  // Just in case. 
  return 0;
}

function handleResize() {
	//var height = windowHeight() - document.getElementById('toolbar').offsetHeight - 45;
	var height = windowHeight() - 170;
	document.getElementById('map').style.height = height + 'px';
}

function toogleText(x) 
{ 
	if (showToogleText[x])
	{
		document.getElementById('toogletext' + x).style.display = 'none';
		showToogleText[x] = false;
	}
	else
	{
		document.getElementById('toogletext' + x).style.display = 'block';
		showToogleText[x] = true;
	}
} 

function saveData(lat, lng) {
	
	var imageSizeX = muestra_size_x;
	var imageSizeY = muestra_size_y;	
	
    //var maxZoom = 9;	
	//128*2^max_zoom--> this originates from the image embed in the tile inserter
	var offsetX = 65536-(imageSizeX/2);
	var offsetY = 65536-(imageSizeY/2);
	var point = projection.fromLatLngToPixel(new GLatLng(lat, lng),maxZoomLevel);
	var	x = point.x - offsetX;
	var	y = point.y - offsetY;	
	//GLog.write('point.x: ' + point.x + '  point.y: ' + point.y);
	
    var url = "phpsql_addmarker.php?id_muestra=" + id_muestra + "&x=" + x + "&y=" + y + "&lat=" + lat + "&lng=" + lng;
    //GLog.write(url);
    GDownloadUrl(url);
  }

//The function to get the various tile images based on the directory.  These file names are in the format produced by the accompanying tile script.
function getTestTileURL(a,b) {
	return baseimageurl+"tiles_test/tile_" + b + "_"+ a.x + "_" + a.y + ".jpg"
}

function getOrigTileURL(a,b) {
	return baseimageurl+"tiles/tile_" + b + "_"+ a.x + "_" + a.y + ".jpg"
}

function getProcessedTileURL(a,b) {
	//if (file_exists($filename))
	return baseimageurl+"tiles_processed/tile_" + b + "_"+ a.x + "_" + a.y + ".jpg"
}

/*
function drawCircleMercator(lat, lng, radius, strokeColor, strokeWidth, strokeOpacity, fillColor, fillOpacity) 
{
  var d2r = Math.PI/180;
  var r2d = 180/Math.PI;
  var Clat = radius * 0.014483;  // Convert statute miles into degrees latitude
  var Clng = Clat/Math.cos(lat*d2r); 
  var Cpoints = []; 
  for (var i=0; i < 33; i++) 
  { 
    var theta = Math.PI * (i/16);
    Cy = lat + (Clat * Math.sin(theta));
    Cx = lng + (Clng * Math.cos(theta));
    var P = new GLatLng(Cy, Cx);
    Cpoints.push(P);
  }

  var polygon = new GPolygon(Cpoints, strokeColor, strokeWidth, strokeOpacity, fillColor, fillOpacity);
  map.addOverlay(polygon);
}
*/

function drawRectangleEuclidean(lat, lng, radius, strokeColor, strokeWidth, strokeOpacity, fillColor, fillOpacity) 
{
	
	var Cpoints = [];
	var point = projection.fromLatLngToPixel(new GLatLng(lat, lng),maxZoomLevel);
	
	var	tl = new GPoint(point.x-960, point.y-960);
	var	tr = new GPoint(point.x+960, point.y-960);
	var	bl = new GPoint(point.x-960, point.y+960);
	var	br = new GPoint(point.x+960, point.y+960);
	
	Cpoints.push(projection.fromPixelToLatLng(bl,maxZoomLevel));
	Cpoints.push(projection.fromPixelToLatLng(br,maxZoomLevel));
	Cpoints.push(projection.fromPixelToLatLng(tr,maxZoomLevel));
	Cpoints.push(projection.fromPixelToLatLng(tl,maxZoomLevel));	
/*	
  var d2r = Math.PI/180;
  var r2d = 180/Math.PI;
  var Clat = radius * 0.014483;  // Convert statute miles into degrees latitude
  var Clng = Clat/Math.cos(lat*d2r); 
   
  for (var i=0; i < 33; i++) 
  { 
    var theta = Math.PI * (i/16);
    Cy = lat + (Clat * Math.sin(theta));
    Cx = lng + (Clng * Math.cos(theta));
    var P = new GLatLng(Cy, Cx);
    Cpoints.push(P);
  }*/

  var polygon = new GPolygon(Cpoints, strokeColor, strokeWidth, strokeOpacity, fillColor, fillOpacity);
  map.addOverlay(polygon);
}


function computeRegions()
{
	roi_finding_param1 = document.getElementById('param1').value;
	roi_finding_param2 = document.getElementById('param2').value;	
	//roi_finding_param3 = document.getElementById('param3').value;
	//roi_finding_param4 = document.getElementById('param4').value;	

	GLog.write('computing regions. params: ' + param1 + ' : ' + param2);
	var result = GDownloadUrl('phpsql_computeregions.php?id_muestra=' + id_muestra + '&param1=' + roi_finding_param1 + '&param2=' + roi_finding_param2
			 + '&param3=' + roi_finding_param3 + '&param4=' + roi_finding_param4, 
			function () {
		      window.location.reload();
		    });
	//window.open('phpsql_computeregions.php?id_muestra=' + id_muestra + '&param1=' + roi_finding_param1 + '&param2=' + roi_finding_param2);
	GLog.write('phpsql_computeregions.php?id_muestra=' + id_muestra + '&param1=' + roi_finding_param1 + '&param2=' + roi_finding_param2
			 + '&param3=' + roi_finding_param3 + '&param4=' + roi_finding_param4);
}

function findNuclei()
{
	GLog.write('finding nuclei in specified regions (not implemented so far)..');
	window.open('phpsql_find_nuclei_in_rois.php?id_muestra=' + id_muestra);
}
function removeMarker(id)
{
	
  GLog.write('removing the marker..');
  GDownloadUrl('phpsql_removemarker.php?id=' + id);
  currentMarker.remove();
}

function confirmMarker(id)
{
//	var blueIcon = new GIcon(G_DEFAULT_ICON);
//	blueIcon.image = "images/markerBlue.png";
//	markerOptions = { icon:blueIcon };
	//currentMarker.
  GLog.write('confirming the marker..');
  GDownloadUrl('phpsql_confirmmarker.php?id=' + id);
  currentMarker.setImage("images/markerBlue.png");  
}

function createMarker(positionX, positionY, id, state, isLL) 
{

  //if the marker is disabled, don't do nothing with it..
  if(state == 2) return;
	
  if(isLL)
	var markerLatLng = new GLatLng(positionX, positionY);
  else
  {
	//var imageSizeX = 92250;
	//var imageSizeY = 81823;
	//var imageSizeX = 78797;
	//var imageSizeY = 81025;
	var imageSizeX = muestra_size_x;
	var imageSizeY = muestra_size_y;	
	
    //var maxZoom = 9;	
	//128*2^max_zoom?
	var offsetX = 65536-(imageSizeX/2);
	var offsetY = 65536-(imageSizeY/2);
	//GLog.write('imageSizeX: ' + imageSizeX + '  imageSizeY: ' + imageSizeY);
	//GLog.write('offsetX: ' + offsetX + '  offsetY: ' + offsetY);
	var point = new GPoint(offsetX+positionX, offsetY+positionY);
	//GLog.write('point.x: ' + point.x + '  point.y: ' + point.y);
	
	var markerLatLng = projection.fromPixelToLatLng(point,maxZoomLevel);

	//GLog.write('markerLatLng.lat(): ' + markerLatLng.lat() + '  markerLatLng.lng(): ' + markerLatLng.lng());
  }
	
  if(state==1)
  {
	var blueIcon = new GIcon(G_DEFAULT_ICON);
	blueIcon.image = "images/markerBlue.png";
	markerOptions = { icon:blueIcon };
  }
  else  markerOptions = {};
  
  var marker = new GMarker(markerLatLng, markerOptions);
  
//	var marker = new GMarker(markerLatLng);
//	if(state==0) marker.setImage("images/markerBlue.png");
		
	//var point = new GLatLng(positionX, positionY);
    //var marker = new GMarker(point);
    GEvent.addListener(marker, "click", function() {

    	//var offset=268435456;
    	var offset=46125;
    	var radius=offset/Math.PI;

    	var lat = marker.getLatLng().lat();
    	var lng = marker.getLatLng().lng();
    	
    	var o1x = 0.703186033;
    	var o2x = 65594.306219867;

    	//var o1y = 0.410137894;
    	//var o2y = 99750.597431181;

    	//lat=73.986237
    	//sin=0.961195457
    	//arg_log=50.5403576
    	//ln=3.922772178
    	//x=0.624328583
    	var o1y = 40911.5;
    	var o2y = 65528.795478211;
    	
    	//var o1y = 46124.999978136;
    	//var o2y = 65594.306219867;


    	var x = Math.round((lng/180 + o1x)*o2x);
  		//var x = Math.round(offset+radius*lng*Math.PI/180);
   		//var y = Math.round(offset-radius*Math.log((1+Math.sin(lat*Math.PI/180))/(1-Math.sin(lat*Math.PI/180)))/2);
    	var y = Math.round(o1y-o2y*Math.log((1+Math.sin(lat*Math.PI/180))/(1-Math.sin(lat*Math.PI/180)))/(2*Math.PI));
    	
        var lngrev = (Math.round(x)/o2x-o1x)*180;
    	var latrev = (Math.PI/2-2*Math.atan(Math.exp((Math.round(y)-o1y)*Math.PI/o2y)))*180/Math.PI;
    	
   		GLog.write('lat: ' + lat + '  lng: ' + lng + '  latrev: ' + latrev + '  lngrev: ' + lngrev + '  x: ' + x + '  y: ' + y);
    	
    	var point = projection.fromLatLngToPixel(marker.getLatLng(),maxZoomLevel);
    	GLog.write('point.x: ' + point.x + '  point.y: ' + point.y);
   		//marker.openInfoWindowHtml('x: ' + x + '  y: ' + y);
   		
    	var message='<a href="javascript:removeMarker(' + id + ')">Remove this marker</a><br/>' + 
    				'<a href="javascript:confirmMarker(' + id + ')">Confirm this marker</a><br/>';

    	currentMarker = marker;
    	
        marker.openInfoWindowHtml(message);
      //GDownloadUrl("phpsql_removemarker.php?id=" + id);
      //marker.remove();      
    });  
    
    //drawCircleMercator(marker.getLatLng().lat(), marker.getLatLng().lng(), 100.0, "#000080", 1, 0.75, "#0000FF",.5);
    //drawRectangleEuclidean(marker.getLatLng().lat(), marker.getLatLng().lng(), 100.0, "#000080", 1, 0.75, "#0000FF",.5);
    map.addOverlay(marker);
    
    //return marker;
  }

//function checkOverview() {
  //    ovmap = ovcontrol.getOverviewMap();
    //  if (ovmap) {      
      //  GLog.write("controls_api has now loaded, so overlayControl.getOverviewMap() = "+overlayControl.getOverviewMap());
//        GLog.write("   and document.getElementById('map_overview') = " + document.getElementById('map_overview'));
  //      GEvent.addListener(overmap,'maptypechanged',function() {
//	  overmap.setMapType(G_NORMAL_MAP);
  //      });
    //    // === The control_api module has not yet loaded, wait again
      //} else {
        //setTimeout("checkOverview()",100);
//      }
  //  }

function init() {
	
  	document.getElementById('param1').value = roi_finding_param1;		  	
	document.getElementById('param2').value = roi_finding_param2;	

	  
	handleResize();
	geocoder = new GClientGeocoder();

	//The copyright for your slide images
	var copyright = new GCopyright(1,
						  new GLatLngBounds(new GLatLng(-90, -180),
											new GLatLng(90, 180)),
						  0,
						  "Copyright 2009");
	
	var copyrightCollection = new GCopyrightCollection("");
	copyrightCollection.addCopyright(copyright);
	
	//projection = new GMercatorProjection(18);
	projection = new EuclideanProjection(18);	
	
	
	//create a custom picture layer
	var tileLayersTest = [ new GTileLayer(copyrightCollection , 1, 17)];
	tileLayersTest[0].getTileUrl = getTestTileURL;
	
	var tileLayersOrig = [ new GTileLayer(copyrightCollection , 1, 17)];
	tileLayersOrig[0].getTileUrl = getOrigTileURL;

	var pic_customMapTest = new GMapType(tileLayersTest, projection, "Test",{maxResolution:maxZoomLevel-1, minResolution:2, errorMessage:"", radius: 1E-3});
	var pic_customMapOriginal = new GMapType(tileLayersOrig, projection, "Original",{maxResolution:maxZoomLevel-1, minResolution:2, errorMessage:"", radius: 1E-3});


//	var tileLayersProcessed = [ new GTileLayer(copyrightCollection , 1, 17)];
//	tileLayersProcessed[0].getTileUrl = getProcessedTileURL;
//	
//	var pic_customMapProcessed = new GMapType(tileLayersProcessed, projection, "Processed",{maxResolution:maxZoomLevel-1, minResolution:2, errorMessage:"", radius: 1E-3});

//	pic_customMapOriginal.hasOverlay = function () {return true;};
	
	//pic_customMapProcessed, 
	//Now create the custom map.
	map = new GMap2(document.getElementById("map"),{mapTypes:[pic_customMapOriginal, pic_customMapTest]});
	var pos = new GControlPosition(G_ANCHOR_TOP_LEFT, new GSize(18,50));

 
	//map.addOverlay(myTileLayer);
	
	//The standard Google navigation controls
	map.addControl(new GLargeMapControl(), pos);
    map.addControl(new GMapTypeControl());	
	
	//Default to a black background just in case tiles are missing
	document.getElementById('map').style.backgroundColor = 'black';
	document.getElementById('columna_der').style.paddingTop = '0';
	document.getElementById('columna_der').style.backgroundColor = 'black';
	
        ovcontrol = new GOverviewMapControl(new GSize(200,200)); 
        map.addControl(ovcontrol); 

        //ovmap = ovcontrol.getOverviewMap(); 
        //ovmap.setZoom(7);
//map.addControl(ov);


      //  ======== Cause the overview to be positioned AFTER IE sets its initial position ======== 
  //    setTimeout("positionOverview(558,254)",1);
      
      //  ======== change the overview map type AFTER the overview finisges initializing =====
//      setTimeout("ovmap.setZoom(7);",1);
//GEvent.addListener(ovmap,"click",function() {alert("you clicked the overview");} );

//setTimeout("checkOverview()",100);

   //     GEvent.addListener(ovmap, "zoomend", function() { ovmap.setZoom(1); });

	////The MINI Map in the lower left corner
	map.addControl(new MiniMapControl());

	var tileLayers2 = [ new GTileLayer('' , 1, 17)];
	tileLayers2[0].getTileUrl = getOrigTileURL;
	var pic_customMiniMap = new GMapType(tileLayers2, new GMercatorProjection(18), "Pic",{maxResolution:0, minResolution:0, errorMessage:"", radius: 1E-3});
	
	var minimap = new GMap2(document.getElementById("minimap"),{mapTypes:[pic_customMiniMap]});
	minimap.disableDoubleClickZoom();
	minimap.disableContinuousZoom();			
	// ====== Handle the Map movements ======    
	var map_moving = 0;
	var minimap_moving = 0;			
	// This function handles what happens when the main map moves
	function Move(){
		minimap_moving = true;
		if (map_moving == false) {
			minimap.setCenter(map.getCenter());
		}
		minimap_moving = false;
	}
	// This function handles what happens when the mini map moves
	function MMove(){
		map_moving = true;
		if (minimap_moving == false) {
			map.setCenter(minimap.getCenter());
		}
		map_moving = false;
	}
	// Listen for when the user moves either map
	GEvent.addListener(map, 'move', Move);
	GEvent.addListener(minimap, 'moveend', MMove);
        GEvent.addListener(map, "click", function(overlay,latlng) {
          if (latlng) {
//            var myHtml = "The GPoint value is: " + map.fromLatLngToDivPixel(latlng) + " at zoom level " + map.getZoom();
            //var myHtml = "You marked this region";
            //map.openInfoWindow(latlng, myHtml);
            //map.addOverlay(new GMarker(latlng));
            map.addOverlay(new GMarker(latlng));
            //drawCircle(latlng.lat(), latlng.lng(), 100.0, "#000080", 1, 0.75, "#0000FF",.5);  
            GLog.write("The GPoint value is: " + map.fromLatLngToDivPixel(latlng) + " at zoom level " + map.getZoom());            
            //var positionX = latlng.lat();
            //var positionY = latlng.lng();
            
            saveData(latlng.lat(), latlng.lng());
            GLog.write("After savedata");
          }
        });

	//Set interaction parameters for the large main map
	map.enableDoubleClickZoom();
	map.enableContinuousZoom();
	map.enableScrollWheelZoom(); 
	map.setCenter(new GLatLng(centerLat, centerLong), initialZoom, pic_customMapOriginal);
	map.setZoom(parseInt(initialZoom));

	//------------------------------------
	 var tileLayerProcessed = new GTileLayer(new GCopyrightCollection(), 1, 17);
	 tileLayerProcessed.getTileUrl = getProcessedTileURL;
	 tileLayerProcessed.getOpacity = function() {return 1.0;} //return 0.5;
	 map.addOverlay(new GTileLayerOverlay(tileLayerProcessed));
	 //-----------------------------
	 
//	var center = createMarker(0, 0, 0, false);
//	map.addOverlay(center);
	
	//setTimeout("map.setZoom(parseInt(initialZoom));",1);
	
    for(i=0;i<markers_x.length;i++) 
    {    	
        //var marker = 
      createMarker(markers_x[i], markers_y[i], markers_id[i], markers_state[i], false);    	
      //GLog.write("Circle Position: " + markers_x[i] + " : " + markers_y[i]);
    };
  

}  //END OF INIT FUNCTION

//MiniMap Div Container
function MiniMapControl() {
}
MiniMapControl.prototype = new GControl();
MiniMapControl.prototype.initialize = function(map) {
	var container = document.createElement("div");
	var minimapDiv = document.createElement("div");
	minimapDiv.id="minimap";
	minimapDiv.style.width = "150px";
	minimapDiv.style.height = "150px";
	minimapDiv.style.border = "thin solid black";
	container.appendChild(minimapDiv);
	map.getContainer().appendChild(container);
	return container;
}
MiniMapControl.prototype.getDefaultPosition = function() {
	return new GControlPosition(G_ANCHOR_BOTTOM_LEFT, new GSize(0, 0));
} 
