<?php 

/******************************************************************************
*
*   This file is part of the software tool "Cervix Image Processing 2 (CIP2)".
*
*   Author: Christoph Bergmeir
*   Supervisor: José M. Benítez
*   Copyright (c) DiCITS Lab, Sci2s group, DECSAI, University of Granada.
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
*   MA  02110-1301, USA.
*
******************************************************************************/

require("bp_apply_filter_gen.php");

function bp_apply_filter($pathData, $filter_name, $use_own_parameter_sets = false, $own_parameter_sets = array(array(0)), $pathData2 = "")
{
	
global $pathCodeOwn, $pathCodeItk, $pathCodeImagemagick, $executableExtension;

$no_parameters = false;
$filter_shortname = "ps";
$outputFilesEnding = ".png";

//$pathData2 = "";

//$parameter_sets = array(array(0));
$parameter_sets = array(array());

switch ($filter_name) {
    case "cvPyrMeanShiftFiltering":
        //$parameter_sets = array(array(10,40,2), array(20,40,2), array(10,40,5), array(20,40,5));
        $parameter_sets = array(array(20,40,0), array(20,40,1), array(20,40,2));
        $filter_shortname = "cvPMS";
        $pathCode = $pathCodeOwn;        
        break;
    case "cvCvtColorBgr2Gray":
        $no_parameters = true;
        $filter_shortname = "cvBgr2Gr";        
        $pathCode = $pathCodeOwn;    	
        break;
    case "cvCvtGray2ColorBgr":
        $no_parameters = true;
        $filter_shortname = "cvGr2Bgr";        
        $pathCode = $pathCodeOwn;    	
        break;
    case "cvCvtColorBgr2Hsv":
        $no_parameters = true;
        $filter_shortname = "cvHsv";        
        $pathCode = $pathCodeOwn;    	
        break;
    case "cvRectBorder":
        $no_parameters = true;
        $filter_shortname = "cvRect";        
        $pathCode = $pathCodeOwn;    	
        break;                 
    case "cvSplit":
        $no_parameters = true;
        $filter_shortname = "cvSplit";        
        $pathCode = $pathCodeOwn;    	
        break;        
    case "cvHoughCircles":
        $parameter_sets = array(array(80,20,10,60), array(80,20,0,100), array(100,20,0,60), array(100,10,0,60));
        $filter_shortname = "cvHCirc";        
        $pathCode = $pathCodeOwn;        
        break;
    case "cvResize":
        $parameter_sets = array(array(256, 256));
        $filter_shortname = "cvResize";        
        $pathCode = $pathCodeOwn;         
        break;
    case "cvCanny":
        $parameter_sets = array(array(50, 150, 3), array(55, 60, 3), array(90, 110, 3));
        $filter_shortname = "cvCanny";        
        $pathCode = $pathCodeOwn;        
        break;             
    case "cvAdaptiveThreshold":
        //$parameter_sets = array(array(0, 121, 20), array(1, 121, 20), array(0, 121, 10), array(1, 121, 10), array(0, 121, 30), array(1, 121, 30) );
        $parameter_sets = array(array(0, 151, 20), array(0, 151, 10), array(0, 151, 30));
        $filter_shortname = "cvATh";        
        $pathCode = $pathCodeOwn;         
        break; 
    case "cvVisResultsRgb":
        $parameter_sets = array(array(0), array(1));
        $filter_shortname = "cvVis";        
        $pathCode = $pathCodeOwn;    
        break;                   
    case "cvROI":
        $parameter_sets = array(array(1080, 600, 870, 1350));
        $filter_shortname = "cvRoi";        
        $pathCode = $pathCodeOwn;         
        break;
    case "cvDilate":
        $parameter_sets = array(array(1));
        $filter_shortname = "cvDil";        
        $pathCode = $pathCodeOwn;         
        break;
    case "cvErode":
        $parameter_sets = array(array(3));
        $filter_shortname = "cvEro";        
        $pathCode = $pathCodeOwn;         
        break;
    case "cvConv":
        $no_parameters = true;
        $filter_shortname = "cvConv";        
        $pathCode = $pathCodeOwn; 
        $outputFilesEnding = ".jpg";        
        break;        
    case "cvHistogramRg":
        $no_parameters = true;
        $filter_shortname = "cvHistRg";        
        $pathCode = $pathCodeOwn;         
        break;
    case "cvHistogramHs":
        $no_parameters = true;
        $filter_shortname = "cvHistHs";        
        $pathCode = $pathCodeOwn;         
        break;        
    case "itkOtsuMultipleThresholdImageFilter":
        $parameter_sets = array(array(2));
        $filter_shortname = "itkOtsuMulti";        
        $pathCode = $pathCodeOwn;         
        break;
    case "itkKappaSigmaThresholdImageFilter":
        $parameter_sets = array(array(0.1, 1));
        $filter_shortname = "itkKappaSigma";        
        $pathCode = $pathCodeOwn;         
        break;        
    case "itkDoubleThresholdImageFilter":
        $parameter_sets = array(array(0, 0, 90, 110), array(0, 0, 100, 115), array(0, 0, 109, 120));
        $filter_shortname = "itkDTh";        
        $pathCode = $pathCodeOwn;         
        break;    
    case "itkThresholdImageFilter":
        $parameter_sets = array(array(0, 220));
        $filter_shortname = "itkTh";        
        $pathCode = $pathCodeOwn;         
        break;
    case "itkCannyEdgeDetectionImageFilter":
        //$parameter_sets = array(array(0.1, 1, 7), array(0.1, 3, 7));
        $parameter_sets = array(array(0.1, 5, 10), array(0.1, 5, 20));
        $filter_shortname = "itkCanny";        
        $pathCode = $pathCodeOwn;         
        break;
    case "itkCannyEdgeDetectionImageFilterOrig":
        //$parameter_sets = array(array(2.0, 1, 9), array(2.0, 3, 9));
        $parameter_sets = array(array(2.0, 5, 10));
        $filter_shortname = "itkOrigCanny";        
        $pathCode = $pathCodeItk;         
        break;    
    case "itkCannySegmentationLevelSetImageFilterNoCannyTest":
        $parameter_sets = array(array(7, 0.1, -1.0, 1.0, -1.0, 128, 300));//Parameters?
        $filter_shortname = "itkCannyLS";        
        $pathCode = $pathCodeOwn;    
        break;             
    case "itkMaskImageFilter":
        $parameter_sets = array(array(0));
        $filter_shortname = "itkMask";        
        $pathCode = $pathCodeOwn;         
        break;    
    case "itkMedianImageFilter":
        //$parameter_sets = array(array(3, 3), array(5, 5), array(7, 7));
        $parameter_sets = array(array(5, 5));
        $filter_shortname = "itkMedian";        
        $pathCode = $pathCodeOwn;         
        break;  
    case "itkClassifyConnectedComponents2":
        $parameter_sets = array(array(255, 15, 75, 1.35));
        $filter_shortname = "itkClass2";        
        $pathCode = $pathCodeOwn;         
        break; 
        
    case "OtsuThresholdImageFilter":
        $parameter_sets = array(array(255, 0));
        $filter_shortname = "itkOtsu";        
        $pathCode = $pathCodeItk;         
        break;
    case "BinaryThresholdImageFilter":
        $parameter_sets = array(array(0, 100, 0, 255));
        $filter_shortname = "itkBTh";        
        $pathCode = $pathCodeItk;         
        break;                         
    case "WatershedSegmentation1":
        //$parameter_sets = array(array(2.0, 10, 0, 0.05, 0));
        //$parameter_sets = array(array(2.0, 10, 0.001, 0.15, 0));
        
        //$parameter_sets = array(array(1.0, 0, 0.01, 0.25, 0));
        $parameter_sets = array(array(1.0, 0, 0.15, 0.3, 0));        
        $filter_shortname = "itkWS";        
        $pathCode = $pathCodeItk;         
        break;    
    case "RGBGradientAnisotropicDiffusionImageFilter":
        $parameter_sets = array(array(5, 0.125));
        $filter_shortname = "itkRGBAniDiff";        
        $pathCode = $pathCodeItk;         
        break; 
    case "CurvatureAnisotropicDiffusionImageFilter":
        $parameter_sets = array(array(5, 0.125, 9.0));
        $filter_shortname = "itkCAniDiff";        
        $pathCode = $pathCodeItk;         
        break; 
    case "GradientMagnitudeRecursiveGaussianImageFilter":
        $parameter_sets = array(array(1.0));
        $filter_shortname = "itkGMRG";        
        $pathCode = $pathCodeItk;         
        break;
    case "GradientMagnitudeImageFilter":
    	$no_parameters = true;
        $filter_shortname = "itkGM";        
        $pathCode = $pathCodeItk;         
        break;
    case "SigmoidImageFilter":
        $parameter_sets = array(array(0.0, 1.0, -0.5, 3.0));
        $filter_shortname = "itkSig";        
        $pathCode = $pathCodeItk;         
        break; 
    case "HoughEllipsesGeodesicActiveContours":
        $parameter_sets = array(array(30, 100, 100, 1000, 10, 10, 70, -10, 20, 0.3, 0.8, 0.8, 2000, 0.6));
        $filter_shortname = "HEGAC";        
        $pathCode = $pathCodeOwn;         
        break; 
    case "FuzzyCMeans":
        //$parameter_sets = array(array(1.0, 1.0), array(2.0, 0.75), array(0.5, 0.5), array(0.75, 2.0), array(2.0, 2.0));
        $parameter_sets = array(array(0.25, 0.25));
        $filter_shortname = "FuzzyCMeans";        
        $pathCode = $pathCodeImagemagick;
        $executableExtension = ".cmd";
        break;  
    case "edisoncmd":
        $parameter_sets = array(array(7, 6.5, 20, 2, 0.3, 0.55), array(4, 60, 150, 4, 0.4, 0.2));
        $filter_shortname = "Edison";        
        $pathCode = $pathCodeOwn;
        $executableExtension = ".cmd";
        break;                
    default:
       die("This filter is not implemented..");
}        

if($use_own_parameter_sets) $parameter_sets = $own_parameter_sets;

$filter_executable = "$pathCode$filter_name$executableExtension";

bp_apply_filter_gen($pathData, $filter_executable, $filter_name, $filter_shortname, $no_parameters, $parameter_sets, $pathData2, $outputFilesEnding);
}
?>
