<?php

/******************************************************************************
*
*   This file is part of the software tool "Cervix Image Processing 2 (CIP2)".
*
*   Author: Christoph Bergmeir
*   Supervisor: José M. Benítez
*   Copyright (c) DiCITS Lab, Sci2s group, DECSAI, University of Granada.
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
*   MA  02110-1301, USA.
*
******************************************************************************/

ini_set('include_path', ini_get('include_path') . PATH_SEPARATOR . $localDocumentRoot);
  
require ("process_large_image.php");

require_once 'Console/Getopt.php';
// Define exit codes for errors   
define('NO_ARGS',10);   
define('INVALID_OPTION',11);   
  
// Reading the incoming arguments - same as $argv   
$args = Console_Getopt::readPHPArgv();   
  
// Make sure we got them (for non CLI binaries)   
if (PEAR::isError($args)) {   
   fwrite(STDERR,$args->getMessage()."\n");   
   exit(NO_ARGS);   
}   
  
// Short options   
//$short_opts = 'ets:';   
$short_opts = 'scr';
  
// Long options   
$long_opts = array(   
   'inSvsFile=',
   'inTiffFile=',   
   'insertRegionsFrom=',
   'writeTilesTo=',    
   'extractRegionsTo=',
   'regionDefinitionFile=',
//   'convertBeforeInsertionTo=',
   );   
  
// Convert the arguments to options - check for the first argument   
if ( realpath($_SERVER['argv'][0]) == __FILE__ ) {   
   $options = Console_Getopt::getOpt($args,$short_opts,$long_opts);   
} else {   
   $options = Console_Getopt::getOpt2($args,$short_opts,$long_opts);   
}   
  
// Check the options are valid   
if (PEAR::isError($options)) {   
   fwrite(STDERR,$options->getMessage()."\n");   
   exit(INVALID_OPTION);   
}   
  
if(sizeof($options[0]) == 0)
{
  die ("Usage:... \n");
//      "--inSvsFile [filename]\n" .
//      "--inTiffFile [filename]\n" .
//      "--insertRegionsFrom [path]\n" .
//      "-e extract regions\n" .
//      "-t generate tiles\n" .
//      "-s shrink input regions\n"      
//      );
};

//print_r($options);

$shrinkTiles = false;
$convertRegions = false;  
$retile = false;

// Write out the contents of the $argv array  
foreach ( $options[0] as $key => $option ) 
{
  echo "Option: ".$option[0]."\n";
  if($option[0] == "--inSvsFile") $inSvsFilename = $option[1];
  if($option[0] == "--inTiffFile") $fullTiffFilename = $option[1];  
  if($option[0] == "--insertRegionsFrom") $pathInsertRegions = $option[1];   
  if($option[0] == "--writeTilesTo") $pathTiles = $option[1];  
  if($option[0] == "--extractRegionsTo") $pathExtractRegions = $option[1];
  if($option[0] == "--regionDefinitionFile") $regionDefinitionFile = $option[1];  
//  if($option[0] == "--convertBeforeInsertionTo") $tmpPathConvertedFiles = $option[1];
//  if($option[0] == "e") $extractRegions = true;
//  if($option[0] == "t") $generateTiles = true;
  if($option[0] == "s") $shrinkTiles = true;
  if($option[0] == "c") $convertRegions = true;  
  if($option[0] == "r") $retile = true;  
  
//  foreach ( $option as $name => $name ) 
//  {
//   fwrite(STDOUT,"$key => $option\n");
//  }  
} 


if(isset($inSvsFilename))
{
  if (!ends_with($inSvsFilename, ".svs") && !ends_with($inSvsFilename, ".SVS")) 
    die("Input file is no svs file..");

  $fullTiffFilename = splitSvsImage($inSvsFilename);
}

//  if(!isset($pathOutput)) die ("Output path has to be specified..");
//  if (!ends_with($pathOutput, "/")) die("outPath has to end with /");

if(isset($fullTiffFilename) && isset($regionDefinitionFile)) extractRegions($fullTiffFilename, $regionDefinitionFile, $pathExtractRegions);

if (isset($pathTiles)) 
{
  if(!isset($pathInsertRegions)) $pathInsertRegions="0";
  generateTiles($fullTiffFilename, $pathTiles, $convertRegions, $pathInsertRegions, $retile, $shrinkTiles);  
}

?>
